/*******************************************************************************

  Intel(R) 82576 Virtual Function Linux driver
  Copyright(c) 1999 - 2008 Intel Corporation.

  This program is free software; you can redistribute it and/or modify it
  under the terms and conditions of the GNU General Public License,
  version 2, as published by the Free Software Foundation.

  This program is distributed in the hope it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  more details.

  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.

  The full GNU General Public License is included in this distribution in
  the file called "COPYING".

  Contact Information:
  Linux NICS <linux.nics@intel.com>
  e1000-devel Mailing List <e1000-devel@lists.sourceforge.net>
  Intel Corporation, 5200 N.E. Elam Young Parkway, Hillsboro, OR 97124-6497

*******************************************************************************/


#include "e1000_hw.h"
#include "e1000_82575.h"


static s32       e1000_init_phy_params_vf(struct e1000_hw *hw);
static s32       e1000_init_nvm_params_vf(struct e1000_hw *hw);
static s32       e1000_init_mac_params_vf(struct e1000_hw *hw);
static s32       e1000_get_bus_info_pcie_vf(struct e1000_hw *hw);
static s32       e1000_check_for_link_vf(struct e1000_hw *hw);
static s32       e1000_get_link_up_info_vf(struct e1000_hw *hw, u16 *speed,
                                              u16 *duplex);
static s32       e1000_setup_link_vf(struct e1000_hw *hw);
static s32       e1000_init_hw_vf(struct e1000_hw *hw);
static s32       e1000_reset_hw_vf(struct e1000_hw *hw);

static void      e1000_release_vf(struct e1000_hw *hw);
static s32       e1000_acquire_vf(struct e1000_hw *hw);
static void      e1000_update_mc_addr_list_vf(struct e1000_hw *hw,
                                              u8 *mc_addr_list,
                                              u32 mc_addr_count,
                                              u32 rar_used_count,
                                              u32 rar_count);

/**
 *  e1000_init_phy_params_vf - Inits PHY params
 *  @hw: pointer to the HW structure
 *
 *  Doesn't do much - there's no PHY available to the VF.
 **/
static s32 e1000_init_phy_params_vf(struct e1000_hw *hw)
{
	DEBUGFUNC("e1000_init_phy_params_vf");
	hw->phy.type = e1000_phy_vf;
	hw->phy.ops.acquire = e1000_acquire_vf;
	hw->phy.ops.release = e1000_release_vf;

	return E1000_SUCCESS;
}

/**
 *  e1000_init_nvm_params_vf - Inits NVM params
 *  @hw: pointer to the HW structure
 *
 *  Doesn't do much - there's no NVM available to the VF.
 **/
static s32 e1000_init_nvm_params_vf(struct e1000_hw *hw)
{
	DEBUGFUNC("e1000_init_nvm_params_vf");
	hw->nvm.type = e1000_nvm_none;
	hw->nvm.ops.acquire = e1000_acquire_vf;
	hw->nvm.ops.release = e1000_release_vf;

	return E1000_SUCCESS;
}

/**
 *  e1000_init_mac_params_vf - Inits MAC params
 *  @hw: pointer to the HW structure
 **/
static s32 e1000_init_mac_params_vf(struct e1000_hw *hw)
{
	struct e1000_mac_info *mac = &hw->mac;

	DEBUGFUNC("e1000_init_mac_params_vf");

	/* Set media type */
	/*
	 * Virtual functions don't care what they're media type is as they
	 * have no direct access to the PHY, or the media.  That is handled
	 * by the physical function driver.
	 */
	hw->phy.media_type = e1000_media_type_unknown;

	/* VF's have no MTA Registers - PF feature only */
	mac->mta_reg_count = 0;
	/* VF's have no access to RAR entries  */
	mac->rar_entry_count = 0;
	/* No ASF features for the VF driver */
	mac->asf_firmware_present = false;
	/* No manageability features for the VF driver */
	mac->arc_subsystem_valid = false;
	/* Disable adaptive IFS mode so the generic funcs don't do anything */
	mac->adaptive_ifs = false;

	/* Function pointers */

	/* bus type/speed/width */
	mac->ops.get_bus_info = e1000_get_bus_info_pcie_vf;
	/* reset */
	mac->ops.reset_hw = e1000_reset_hw_vf;
	/* hw initialization */
	mac->ops.init_hw = e1000_init_hw_vf;
	/* link setup */
	mac->ops.setup_link = e1000_setup_link_vf;
	/* check for link */
	mac->ops.check_for_link = e1000_check_for_link_vf;
	/* link info */
	mac->ops.get_link_up_info = e1000_get_link_up_info_vf;

	/* multicast address update */
	mac->ops.update_mc_addr_list = e1000_update_mc_addr_list_vf;


	return E1000_SUCCESS;
}


/**
 *  e1000_acquire_vf - Acquire rights to access PHY or NVM.
 *  @hw: pointer to the HW structure
 *
 *  There is no PHY or NVM so we want all attempts to acquire these to fail.
 *  In addition, the MAC registers to access PHY/NVM don't exist so we don't
 *  even want any SW to attempt to use them.
 **/
static s32 e1000_acquire_vf(struct e1000_hw *hw)
{
	return -E1000_ERR_PHY;
}

/**
 *  e1000_release_vf - Release PHY or NVM
 *  @hw: pointer to the HW structure
 *
 *  There is no PHY or NVM so we want all attempts to acquire these to fail.
 *  In addition, the MAC registers to access PHY/NVM don't exist so we don't
 *  even want any SW to attempt to use them.
 **/
static void e1000_release_vf(struct e1000_hw *hw)
{
	return;
}

/**
 *  e1000_init_function_pointers_vf - Inits function pointers
 *  @hw: pointer to the HW structure
 **/
void e1000_init_function_pointers_vf(struct e1000_hw *hw)
{
	DEBUGFUNC("e1000_init_function_pointers_vf");

	hw->mac.ops.init_params = e1000_init_mac_params_vf;
	hw->nvm.ops.init_params = e1000_init_nvm_params_vf;
	hw->phy.ops.init_params = e1000_init_phy_params_vf;
}

/**
 *  e1000_get_bus_info_pcie_vf - Gets the bus info.
 *  @hw: pointer to the HW structure
 *
 *  Virtual functions are not really on their own bus.
 **/
static s32 e1000_get_bus_info_pcie_vf(struct e1000_hw *hw)
{
	struct e1000_bus_info *bus = &hw->bus;

	DEBUGFUNC("e1000_get_bus_info_pcie_vf");

	/* Do not set type PCI-E because we don't want disable master to run */
	bus->type = e1000_bus_type_reserved;
	bus->speed = e1000_bus_speed_2500;

	return 0;
}

/**
 *  e1000_setup_link_vf - Sets up link.
 *  @hw: pointer to the HW structure
 *
 *  Virtual functions cannot change link.
 **/
static s32 e1000_setup_link_vf(struct e1000_hw *hw)
{
	DEBUGFUNC("e1000_setup_link_vf");

	return E1000_SUCCESS;
}

/**
 *  e1000_get_link_up_info_vf - Gets link info.
 *  @hw: pointer to the HW structure
 *  @speed: pointer to 16 bit value to store link speed.
 *  @duplex: pointer to 16 bit value to store duplex.
 *
 *  Since we cannot read the PHY and get accurate link info, we must rely upon
 *  the status register's data which is often stale and inaccurate.
 **/
static s32 e1000_get_link_up_info_vf(struct e1000_hw *hw, u16 *speed,
                                     u16 *duplex)
{
	s32 ret_val;

	DEBUGFUNC("e1000_get_link_up_info_vf");

	ret_val = e1000_get_speed_and_duplex_copper_generic(hw, speed,
	                                                    duplex);
	return ret_val;
}

/**
 *  e1000_check_for_link_vf - Checks for link
 *  @hw: pointer to the HW structure
 *
 *  Since we cannot do anything with link configuration, we just set the
 *  get_link_status to false and exit.
 **/
static s32 e1000_check_for_link_vf(struct e1000_hw *hw)
{
	DEBUGFUNC("e1000_check_for_link_vf");
	hw->mac.get_link_status = false;

	return E1000_SUCCESS;
}

/**
 *  e1000_reset_hw_vf - Resets the HW
 *  @hw: pointer to the HW structure
 *
 *  VF's provide a function level reset. This is done using bit 26 of ctrl_reg.
 *  This is all the reset we can perform on a VF.
 **/
static s32 e1000_reset_hw_vf(struct e1000_hw *hw)
{
	u32 ctrl;

	DEBUGFUNC("e1000_reset_hw_vf");

	ctrl = E1000_READ_REG(hw, E1000_CTRL);

	DEBUGOUT("Issuing a function level reset to MAC\n");
	E1000_WRITE_REG(hw, E1000_CTRL, ctrl | E1000_CTRL_RST);
	usec_delay(100);

	return E1000_SUCCESS;
}

/**
 *  e1000_init_hw_vf - Inits the HW
 *  @hw: pointer to the HW structure
 *
 *  Not much to do here except clear the PF Reset indication if there is one.
 **/
static s32 e1000_init_hw_vf(struct e1000_hw *hw)
{
	u32 v2p_mailbox = E1000_READ_REG(hw, E1000_V2PMAILBOX(0));
	s32 ret_val = E1000_SUCCESS;

	DEBUGFUNC("e1000_init_hw_vf");

        /* Check for RSTI and take steps to clear... */
        if (v2p_mailbox & E1000_V2PMAILBOX_RSTI) {
		v2p_mailbox = E1000_READ_REG(hw, E1000_V2PMAILBOX(0));
		if ((v2p_mailbox & E1000_V2PMAILBOX_RSTI) != 0) {
			/* Error Condition */
			ret_val = -1;
            }
        }

	return ret_val;
}

/**
 *  e1000_update_mc_addr_list_vf - Update Multicast addresses
 *  @hw: pointer to the HW structure
 *  @mc_addr_list: array of multicast addresses to program
 *  @mc_addr_count: number of multicast addresses to program
 *  @rar_used_count: the first RAR register free to program
 *  @rar_count: total number of supported Receive Address Registers
 *
 *  Updates the Receive Address Registers and Multicast Table Array.
 *  The caller must have a packed mc_addr_list of multicast addresses.
 *  The parameter rar_count will usually be hw->mac.rar_entry_count
 *  unless there are workarounds that change this.
 **/
void e1000_update_mc_addr_list_vf(struct e1000_hw *hw,
                                  u8 *mc_addr_list, u32 mc_addr_count,
                                  u32 rar_used_count, u32 rar_count)
{
	u32 msgbuf[E1000_VFMAILBOX_SIZE];
	u32 cnt;
	u32 i;
	u8 *p;

	DEBUGFUNC("e1000_update_mc_addr_list_vf");

	/* Each entry in the list uses 1.5 32 bit words.  We have 15
	 * 32 bit words available in our HW msg buffer (minus 1 for the
	 * msg type).  That's 10 addresses if we pack 'em right.  If
	 * there are more than 10 MC addresses to add then punt the
	 * extras for now and then add code to handle more than 10 later.
	 * It would be unusual for a server to request that many multi-cast
	 * addresses except for in large enterprise network environments.
	 */

	DEBUGOUT1("MC Addr Count = %d\n", mc_addr_count);

	cnt = (mc_addr_count > 10) ? 10 : mc_addr_count;
	msgbuf[0] = E1000_VF_SET_MULTICAST;
	msgbuf[0] |= cnt << E1000_VT_MSGINFO_SHIFT;
	p = (u8 *)&msgbuf[1];

	for (i = 0; i < cnt; i++) {
		memcpy(p, &mc_addr_list[i * 6], 6);
		p += 6;
	}

	e1000_send_mail_to_pf_vf(hw, msgbuf, E1000_VFMAILBOX_SIZE);
}

/**
 *  e1000_send_mail_to_pf_vf - Sends a mailbox message from VF to PF
 *  @hw: pointer to the HW structure
 *  @msg: The message buffer
 *  @size: Length of buffer
 **/
s32 e1000_send_mail_to_pf_vf(struct e1000_hw *hw, u32 *msg, s16 size)
{
	u32 v2p_mailbox = E1000_READ_REG(hw, E1000_V2PMAILBOX(0));
	s32 ret_val = E1000_SUCCESS;
	s16 i;

	/*
	 * What happens if you enter this function and PFSTS is set?  From
	 * my thinking it should be an error, but obviously that means an
	 * event was missed and the message should be stored somewhere.
	 * but where?  Or maybe it should just be dropped on the floor?
	 * I'll have to think 'pon it some more
	 */

	/*
	 * if any of the indicated bits are set then there has been a
	 * programming error or the PF has been reset underneath the feet
	 * of the VF
	 */
	if (v2p_mailbox & (E1000_V2PMAILBOX_PFU |
	                   E1000_V2PMAILBOX_PFSTS |
	                   E1000_V2PMAILBOX_RSTI)) {
		ret_val = -1;
		goto out;
	} else {
		/* Take ownership of the buffer */
		v2p_mailbox |= E1000_V2PMAILBOX_VFU;
		E1000_WRITE_REG(hw, E1000_V2PMAILBOX(0), v2p_mailbox);
		/* Make sure we have ownership now... */
		if (v2p_mailbox & E1000_V2PMAILBOX_PFU) {
			/*
			 * oops, PF grabbed ownership while we were attempting
			 * to take ownership - avoid the race condition
			 */
			ret_val = -1;
			goto out;
		}
	}

	/*
	 * At this point we have established ownership of the VF mailbox memory
	 * buffer.  IT IS IMPORTANT THAT THIS OWNERSHIP BE GIVEN UP!  Whether
	 * success or failure, the VF ownership bit must be cleared before
	 * exiting this function - so if you change this function keep that
	 * in mind
	 */

	/* check for overflow */
	if (size > E1000_VFMAILBOX_SIZE) {
		ret_val = -1;
		goto out;
	}

	/*
	 * copy the caller specified message to the mailbox
	 * memory buffer
	 */
	for (i = 0; i < size; i++) {
		E1000_WRITE_REG(hw, E1000_VFVMBMEM(i * 4), msg[i]);
	}

	/* Interrupt the PF to tell it a message has been sent */
	/*
	 * The PF request bit is write only and will be read back
	 * as zero on the next register read below - thus no need
	 * to programmatically clear it
	 */
	v2p_mailbox |= E1000_V2PMAILBOX_REQ;
	E1000_WRITE_REG(hw, E1000_V2PMAILBOX(0), v2p_mailbox);

out:
	/* Clear the VF mail box memory buffer ownership bit */
	v2p_mailbox &= ~(E1000_V2PMAILBOX_VFU | E1000_V2PMAILBOX_REQ);

	/* Clear VF ownership of the mail box memory buffer */
	E1000_WRITE_REG(hw, E1000_V2PMAILBOX(0), v2p_mailbox);

	return ret_val;
}

/**
 *  e1000_check_for_pf_ack_vf - checks to see if the PF has ACK'd
 *  @hw: pointer to the HW structure
 *
 *  returns true if the PF has set the ACK bit or else false
 **/
bool e1000_check_for_pf_ack_vf(struct e1000_hw *hw)
{
	u32 v2p_mailbox = E1000_READ_REG(hw, E1000_V2PMAILBOX(0));
	bool ret_val = false;

	if ((v2p_mailbox & E1000_V2PMAILBOX_PFACK)) {
		ret_val = true;
	}

	return ret_val;
}

/**
 *  e1000_receive_mail_from_pf_vf - Receives a mailbox message from PF to VF
 *  @hw: pointer to the HW structure
 *  @msg: The message buffer
 *  @size: Length of buffer
 **/
s32 e1000_receive_mail_from_pf_vf(struct e1000_hw *hw, u32 *msg, s16 size)
{
	u32 v2p_mailbox = E1000_READ_REG(hw, E1000_V2PMAILBOX(0));
	s32 ret_val = E1000_SUCCESS;
	s16 i;

	/*
	 * copy the caller specified message to the mailbox
	 * memory buffer
	 */
	for (i = 0; i < size; i++) {
		msg[i] = E1000_READ_REG(hw, E1000_VFVMBMEM(i * 4));
	}

	/*
	 * Acknowledge receipt of the message to the PF and then
	 * we're done
	 */
	/* clear status indication bit */
	v2p_mailbox &= ~(E1000_V2PMAILBOX_PFSTS | E1000_V2PMAILBOX_VFU);
	v2p_mailbox |= E1000_V2PMAILBOX_ACK;	    /* Set PF Ack bit */
	E1000_WRITE_REG(hw, E1000_V2PMAILBOX(0), v2p_mailbox);

	return ret_val;
}

/**
 *  e1000_check_for_pf_mail_vf - checks to see if the PF has sent mail
 *  @hw: pointer to the HW structure
 *
 *  returns true if the PF has set the Status bit or else false
 **/
bool e1000_check_for_pf_mail_vf(struct e1000_hw *hw, u32 *vf_mb_val)
{
	u32 v2p_mailbox = E1000_READ_REG(hw, E1000_V2PMAILBOX(0));
	bool ret_val = false;

	if ((v2p_mailbox & E1000_V2PMAILBOX_PFSTS)) {
		ret_val = true;
	}

	*vf_mb_val = v2p_mailbox;

	return ret_val;
}

/**
 *  e1000_send_mail_to_vf - Sends a mailbox message from PF to VF
 *  @hw: pointer to the HW structure
 *  @msg: The message buffer
 *  @vf_number: the VF index
 *  @size: Length of buffer
 **/
s32 e1000_send_mail_to_vf(struct e1000_hw *hw, u32 *msg, u32 vf_number,
                          s16 size)
{
	u32 p2v_mailbox = E1000_READ_REG(hw, E1000_P2VMAILBOX(vf_number));
	s32 ret_val = E1000_SUCCESS;
	s16 i;

	/*
	 * if the VF owns the mailbox then we can't grab the mailbox buffer
	 * - mostly an indication of a programming error
	 */
	if (p2v_mailbox & E1000_P2VMAILBOX_VFU) {
		ret_val = -1;
		goto out_no_write;
	} else {
		/* Take ownership of the buffer */
		p2v_mailbox |= E1000_P2VMAILBOX_PFU;
		E1000_WRITE_REG(hw, E1000_P2VMAILBOX(vf_number), p2v_mailbox);
		p2v_mailbox = E1000_READ_REG(hw, E1000_P2VMAILBOX(vf_number));
		/* Make sure we have ownership now... */
		if (p2v_mailbox & E1000_P2VMAILBOX_VFU) {
			/*
			 * oops, VF grabbed ownership while we were attempting
			 * to take ownership - avoid the race condition
			 */
			ret_val = -2;
			goto out_no_write;
		}
	}

	/*
	 * At this point we have established ownership of the PF mailbox memory
	 * buffer.  IT IS IMPORTANT THAT THIS OWNERSHIP BE GIVEN UP!  Whether
	 * success or failure, the PF ownership bit must be cleared before
	 * exiting this function - so if you change this function keep that
	 * in mind
	 */

	/* Clear PF ownership of the mail box memory buffer */
	/*
	 * Do this whether success or failure on the wait for ack from
	 * the PF
	 */
	p2v_mailbox &= ~(E1000_P2VMAILBOX_PFU);

	/* check for overflow */
	if (size > E1000_VFMAILBOX_SIZE) {
		ret_val = -3;
		goto out;
	}

	/*
	 * copy the caller specified message to the mailbox
	 * memory buffer
	 */
	for (i = 0; i < size; i++) {
		E1000_WRITE_REG(hw,
				((E1000_VMBMEM(vf_number)) + (i * 4)), msg[i]);
	}

	/* Interrupt the VF to tell it a message has been sent */
	p2v_mailbox |= E1000_P2VMAILBOX_STS;

out:
	E1000_WRITE_REG(hw, E1000_P2VMAILBOX(vf_number), p2v_mailbox);

out_no_write:
	return ret_val;

}

/**
 *  e1000_receive_mail_from_vf - Receives a mailbox message from VF to PF
 *  @hw: pointer to the HW structure
 *  @msg: The message buffer
 *  @vf_number: the VF index
 *  @size: Length of buffer
 **/
s32 e1000_receive_mail_from_vf(struct e1000_hw *hw,
                               u32 *msg, u32 vf_number, s16 size)
{
	u32 p2v_mailbox = E1000_READ_REG(hw, E1000_P2VMAILBOX(vf_number));
	s16 i;

	/*
	 * Should we be checking if the VF has set the ownership bit?
	 * I don't know... presumably well written software will set the
	 * VF mailbox memory ownership bit but I can't think of a reason
	 * to call it an error if it doesn't... I'll think 'pon it some more
	 */

	/*
	 * No message ready polling mechanism - the presumption is that
	 * the caller knows there is a message because of the interrupt
	 * ack
	 */

	/*
	 * copy the caller specified message to the mailbox
	 * memory buffer
	 */
	for (i = 0; i < size; i++) {
		msg[i] = E1000_READ_REG(hw,
					((E1000_VMBMEM(vf_number)) + (i * 4)));
	}

	/*
	 * Acknowledge receipt of the message to the VF and then
	 * we're done
	 */
	p2v_mailbox |= E1000_P2VMAILBOX_ACK;	    /* Set PF Ack bit */
	E1000_WRITE_REG(hw, E1000_P2VMAILBOX(vf_number), p2v_mailbox);

	return E1000_SUCCESS;	/* Success is the only option  */
}

/**
 *  e1000_vmdq_loopback_enable_vf - Enables VM to VM queue loopback replication
 *  @hw: pointer to the HW structure
 **/
void e1000_vmdq_loopback_enable_vf(struct e1000_hw *hw)
{
	u32 reg;

	reg = E1000_READ_REG(hw, E1000_DTXSWC);
	reg |= E1000_DTXSWC_VMDQ_LOOPBACK_EN;
	E1000_WRITE_REG(hw, E1000_DTXSWC, reg);
}

/**
 *  e1000_vmdq_loopback_disable_vf - Disable VM to VM queue loopbk replication
 *  @hw: pointer to the HW structure
 **/
void e1000_vmdq_loopback_disable_vf(struct e1000_hw *hw)
{
	u32 reg;

	reg = E1000_READ_REG(hw, E1000_DTXSWC);
	reg &= ~(E1000_DTXSWC_VMDQ_LOOPBACK_EN);
	E1000_WRITE_REG(hw, E1000_DTXSWC, reg);
}

/**
 *  e1000_vmdq_replication_enable_vf - Enable replication of brdcst & multicst
 *  @hw: pointer to the HW structure
 *
 *  Enables replication of broadcast and multicast packets from the network
 *  to VM's which have their respective broadcast and multicast accept
 *  bits set in the VM Offload Register.  This gives the PF driver per
 *  VM granularity control over which VM's get replicated broadcast traffic.
 **/
void e1000_vmdq_replication_enable_vf(struct e1000_hw *hw, u32 enables)
{
	u32 reg;
	u32 i;

	for (i = 0; i < MAX_NUM_VFS; i++) {
		if (enables & (1 << i)) {
			reg = E1000_READ_REG(hw, E1000_VMOLR(i));
			reg |= (E1000_VMOLR_AUPE |
				E1000_VMOLR_BAM |
				E1000_VMOLR_MPME);
			E1000_WRITE_REG(hw, E1000_VMOLR(i), reg);
		}
	}

	reg = E1000_READ_REG(hw, E1000_VT_CTL);
	reg |= E1000_VT_CTL_VM_REPL_EN;
	E1000_WRITE_REG(hw, E1000_VT_CTL, reg);
}

/**
 *  e1000_vmdq_replication_disable_vf - Disable replication of brdcst & multicst
 *  @hw: pointer to the HW structure
 *
 *  Disables replication of broadcast and multicast packets to the VM's.
 **/
void e1000_vmdq_replication_disable_vf(struct e1000_hw *hw)
{
	u32 reg;

	reg = E1000_READ_REG(hw, E1000_VT_CTL);
	reg &= ~(E1000_VT_CTL_VM_REPL_EN);
	E1000_WRITE_REG(hw, E1000_VT_CTL, reg);
}

/**
 *  e1000_vmdq_enable_replication_mode_vf - Enables replication mode in the device
 *  @hw: pointer to the HW structure
 **/
void e1000_vmdq_enable_replication_mode_vf(struct e1000_hw *hw)
{
	u32 reg;

	reg = E1000_READ_REG(hw, E1000_VT_CTL);
	reg |= E1000_VT_CTL_VM_REPL_EN;
	E1000_WRITE_REG(hw, E1000_VT_CTL, reg);
}

/**
 *  e1000_vmdq_broadcast_replication_enable_vf - Enable replication of brdcst
 *  @hw: pointer to the HW structure
 *  @enables: PoolSet Bit - if set to ALL_QUEUES, apply to all pools.
 *
 *  Enables replication of broadcast packets from the network
 *  to VM's which have their respective broadcast accept
 *  bits set in the VM Offload Register.  This gives the PF driver per
 *  VM granularity control over which VM's get replicated broadcast traffic.
 **/
void e1000_vmdq_broadcast_replication_enable_vf(struct e1000_hw *hw,
						u32 enables)
{
	u32 reg;
	u32 i;

	for (i = 0; i < MAX_NUM_VFS; i++) {
		if ((enables == ALL_QUEUES) || (enables & (1 << i))) {
			reg = E1000_READ_REG(hw, E1000_VMOLR(i));
			reg |= E1000_VMOLR_BAM;
			E1000_WRITE_REG(hw, E1000_VMOLR(i), reg);
		}
	}
}

/**
 *  e1000_vmdq_multicast_replication_enable_vf - Enable replication of multicast
 *  @hw: pointer to the HW structure
 *  @enables: PoolSet Bit - if set to ALL_QUEUES, apply to all pools.
 *
 *  Enables replication of multicast packets from the network
 *  to VM's which have their respective multicast promiscuous mode enable
 *  bits set in the VM Offload Register.  This gives the PF driver per
 *  VM granularity control over which VM's get replicated multicast traffic.
 **/
void e1000_vmdq_multicast_replication_enable_vf(struct e1000_hw *hw,
						u32 enables)
{
	u32 reg;
	u32 i;

	for (i = 0; i < MAX_NUM_VFS; i++) {
		if ((enables == ALL_QUEUES) || (enables & (1 << i))) {
			reg = E1000_READ_REG(hw, E1000_VMOLR(i));
			reg |= E1000_VMOLR_MPME;
			E1000_WRITE_REG(hw, E1000_VMOLR(i), reg);
		}
	}
}

/**
 *  e1000_vmdq_broadcast_replication_disable_vf - Disable replication
 *  of broadcast
 *  @hw: pointer to the HW structure
 *  @disables: PoolSet Bit - if set to ALL_QUEUES, apply to all pools.
 *
 *  Disables replication of broadcast packets for specific pools. If
 *  replication (bc/mc) is disabled on all pools then replication mode is
 *  turned off.
 **/
void e1000_vmdq_broadcast_replication_disable_vf(struct e1000_hw *hw,
						u32 disables)
{
	u32 reg;
	u32 i;
	u32 oneenabled = 0;

	for (i = 0; i < MAX_NUM_VFS; i++) {
		reg = E1000_READ_REG(hw, E1000_VMOLR(i));
		if ((disables == ALL_QUEUES) || (disables & (1 << i))) {
			reg &= ~(E1000_VMOLR_BAM);
			E1000_WRITE_REG(hw, E1000_VMOLR(i), reg);
		}
		if (!oneenabled && (reg & (E1000_VMOLR_AUPE |
				E1000_VMOLR_BAM |
				E1000_VMOLR_MPME)))
				oneenabled = 1;
	}
	if (!oneenabled) {
		reg = E1000_READ_REG(hw, E1000_VT_CTL);
		reg &= ~(E1000_VT_CTL_VM_REPL_EN);
		E1000_WRITE_REG(hw, E1000_VT_CTL, reg);
	}
}

/**
 *  e1000_vmdq_multicast_replication_disable_vf - Disable replication
 *  of multicast packets
 *  @hw: pointer to the HW structure
 *  @disables: PoolSet Bit - if set to ALL_QUEUES, apply to all pools.
 *
 *  Disables replication of multicast packets for specific pools. If
 *  replication (bc/mc) is disabled on all pools then replication mode is
 *  turned off.
 **/
void e1000_vmdq_multicast_replication_disable_vf(struct e1000_hw *hw,
						u32 disables)
{
	u32 reg;
	u32 i;
	u32 oneenabled = 0;

	for (i = 0; i < MAX_NUM_VFS; i++) {
		reg = E1000_READ_REG(hw, E1000_VMOLR(i));
		if ((disables == ALL_QUEUES) || (disables & (1 << i))) {
			reg &= ~(E1000_VMOLR_MPME);
			E1000_WRITE_REG(hw, E1000_VMOLR(i), reg);
		}
		if (!oneenabled && (reg & (E1000_VMOLR_AUPE |
				E1000_VMOLR_BAM |
				E1000_VMOLR_MPME)))
				oneenabled = 1;
	}

	if (!oneenabled) {
		reg = E1000_READ_REG(hw, E1000_VT_CTL);
		reg &= ~(E1000_VT_CTL_VM_REPL_EN);
		E1000_WRITE_REG(hw, E1000_VT_CTL, reg);
	}
}

